-- 3D Drucker using https://minecraftshapes.com/
-------------------------------------------------


basic = require("lib.basic")




local function createMatrix(filePath, dim)
  --create empty matrix
  matrix = {}
  for i = 1, dim do
    matrix[i] = {}
    for j = 1, dim do
      matrix[i][j] = 0
    end
  end

  --read shape svg
  local lines = {}
  for line in io.lines(filePath) do 
    lines[#lines + 1] = line
  end

  --write 1 to each square that is set in the file
  for k,v in pairs(lines) do
    print(k)
    if string.match(v, "green") then
      --print(k .. ": yay")
      -- lines containing "green"
      xStart = string.find(v, "x=\"")
      xEnd = string.find(v, "\" y=")
      yStart = string.find(v, "y=\"")
      yEnd = string.find(v, "\" st")
  
      xStr = string.sub(v, xStart+3, xEnd-1)
      yStr = string.sub(v, yStart+3, yEnd-1)
  
      x = tonumber(xStr) / 18
      y = tonumber(yStr) / 18
  
      print(x .. " " .. y)
      matrix[x][y] = 1
    end
  end
  return matrix
end


local function itemInInventory()
  for i=1, 16 do
    turtle.select(i)
    itemCount = turtle.getItemCount()
    if itemCount > 0 then
      return true
    end
  end
  print("No material. Press Return to continue")
  read()
  itemInInventory()
end




local function countBlocks(matrix)
  local counter = 0
  for i=1, #matrix do
    for j=1, #matrix[i] do
      if matrix[i][j] == 1 then
        counter = counter + 1
      end
    end
  end
  return counter
end


local function confirmationDialog(input)
  if input == "y" or input == "yes" then
    return true
  elseif input == "n" or input == "no" then
    return false
  else
    print("confirmationDialogError")
    return "Error"
  end
end

local function printLayer(matrix, withChest)
  --for i=1, #matrix do
  for i=#matrix, 1, -1 do
    sleep(0.5)
    for j=1, #matrix[i] do
      --print(i .. "," .. j .. ": " .. matrix[i][j])
      --sleep(1)
      turtle.forward()
      if matrix[i][j] == 1 then
        itemInInventory()
        turtle.placeDown()
      end
    end
    for j=1, #matrix[i] do
      turtle.back()
    end
    turtle.turnRight()
    turtle.forward()
    turtle.turnLeft()
  end
  turtle.turnRight()

  -- go back to start
  for i=#matrix, 1, -1 do
    turtle.back()
  end
  turtle.turnLeft()
end

local function printLayers(matrix, withChest, height)
  for i=1, height do
    printLayer(matrix, withChest)
    turtle.up()
  end
end

function main()
  print("Print with Chest? [yn]")
  local withChest = confirmationDialog(read())

  print("Filepath? (/brulijam/files/shape.svg)")
  local filePath = read()

  print("Dimension of the shape?")
  local dim = read()

  matrix = createMatrix(filePath, dim)

  local blocksPerLayer = countBlocks(matrix)

  print("How many Layers?")
  local height = read()

  local blocksForAllLayers = blocksPerLayer * height

  print("One Layer would need " .. blocksPerLayer .. " Blocks.")
  print(height .. " Layers would need " .. blocksForAllLayers .. " Blocks (" .. blocksForAllLayers/64 .. " Stacks)")
  print("Continue? [yn]")

  local startPrinting = confirmationDialog(read())

  if startPrinting then
    printLayers(matrix, withChest, height)
    --printLayer(matrix, withChest)
  end
end

main()

